﻿using System.Windows;
using System.Windows.Media;
using System.Globalization;

/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

namespace Bio.Views.Alignment.Views
{
    public class HeaderRulerIndex : FrameworkElement
    {
        public static readonly DependencyProperty DisplayIndexProperty = DependencyProperty.Register("DisplayIndex", typeof(int),typeof(HeaderRulerIndex),
                                                  new FrameworkPropertyMetadata(0, 
                                                      FrameworkPropertyMetadataOptions.AffectsRender |
                                                      FrameworkPropertyMetadataOptions.AffectsMeasure));

        public static readonly DependencyProperty ColumnProperty = DependencyProperty.Register("Column", typeof(int), typeof(HeaderRulerIndex),
                                                  new FrameworkPropertyMetadata(0,
                                                      FrameworkPropertyMetadataOptions.AffectsRender |
                                                      FrameworkPropertyMetadataOptions.AffectsMeasure));

        public static readonly DependencyProperty ForegroundProperty = DependencyProperty.Register("Foreground", typeof(Brush), typeof(HeaderRulerIndex), 
                                                    new FrameworkPropertyMetadata(null));

        private FormattedText _text;

        public int DisplayIndex
        {
            get { return (int) base.GetValue(DisplayIndexProperty);  }   
            set { base.SetValue(DisplayIndexProperty, value);}
        }

        public int Column
        {
            get { return (int)base.GetValue(ColumnProperty); }
            set { base.SetValue(ColumnProperty, value); }
        }

        public Brush Foreground
        {
            get { return (Brush)GetValue(ForegroundProperty); }
            set { SetValue(ForegroundProperty, value); }
        }

        protected override Geometry GetLayoutClip(Size layoutSlotSize)
        {
            return null;
        }

        protected override Size MeasureOverride(Size availableSize)
        {
            string columnNumber = (Column + DisplayIndex).ToString();
            if (DisplayIndex == 1 || DisplayIndex > 0 && DisplayIndex % 10 == 0)
            {
                _text = new FormattedText(columnNumber, CultureInfo.CurrentUICulture,
                                             FlowDirection.LeftToRight,
                                             new Typeface("Arial"), 12, Foreground);
                return new Size(_text.Width, _text.Height);
            }
            return new Size(20,20);
        }

        protected override void  OnRender(DrawingContext dc)
        {
            string columnNumber = (Column + DisplayIndex).ToString();
            ToolTip = columnNumber;

            if (DisplayIndex == 1 || DisplayIndex > 0 && DisplayIndex % 10 == 0)
            {
                dc.DrawText(_text, new Point(10 - _text.Width / 2, 2));
                dc.DrawLine(new Pen(Foreground,1), new Point(9, _text.Height+3), new Point(9,20));
            }
            else
            {
                dc.DrawRectangle(Brushes.Transparent, null, new Rect(0, 0, DesiredSize.Width, DesiredSize.Height));
            }
        }
    }
}
