﻿/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Threading;
using Bio.Data.Providers.Interfaces;
using Bio.Views.Interfaces;

namespace BioBrowser.Models
{
    /// <summary>
    /// This class is used to hold the located data providers
    /// </summary>
    internal class Extensions
    {
        /// <summary>
        /// Our data providers
        /// </summary>
        [ImportMany]
        public IEnumerable<IBioDataProvider> DataProviders { get; set; }

        /// <summary>
        /// The loaded visualizations
        /// </summary>
        [ImportMany]
        public IEnumerable<IBioViewProvider> Visualizations { get; set; }

        /// <summary>
        /// Singleton access
        /// </summary>
        static LazyInit<Extensions> _instance = new LazyInit<Extensions>(() => new Extensions(), LazyInitMode.EnsureSingleExecution);
        public static Extensions Current
        {
            get { return _instance.Value; }            
        }

        private Extensions()
        {
        }
    }
}
