﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;

namespace Bio.Data.Providers.rCAD.Structure.Models
{
    public class SequenceEntry : KeyNameData
    {
        public event SelectionStateChangedEventHandler SelectionStateChanged;
        public delegate void SelectionStateChangedEventHandler(object sender, EventArgs e);

        private bool _isSelected;
        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                _isSelected = value;
                if (!_data.SelectedSet.Contains(Id) && _isSelected)
                    _data.SelectedSet.Add(Id);

                if (_data.SelectedSet.Contains(Id) && !_isSelected)
                    _data.SelectedSet.Remove(Id);

                if (SelectionStateChanged != null)
                {
                    SelectionStateChanged(this, EventArgs.Empty); //Fire a notification
                }

                OnPropertyChanged("IsSelected");
            }
        }

        private bool _hasRendering;
        public bool HasRendering
        {
            get { return _hasRendering; }
            set
            {
                _hasRendering = value;
                OnPropertyChanged("HasRendering");
            }
        }

        public SequenceEntry(StructureModelSet data)
        {
            _data = data;
        }

        private StructureModelSet _data;
    }
}
