﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Collections.Generic;

namespace Bio.Data.Providers.rCAD.Structure.Models
{
    public class StructureModelSet
    {
        /// <summary>
        /// The list of SeqID's selected
        /// </summary>
        public List<int> SelectedSet { get; private set; }

        /// <summary>
        /// The name of the secondary structure set.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// The alignment id of the associated secondary structure models in the set
        /// </summary>
        public int AlignmentId { get; set; }

        /// <summary>
        /// This flag indicates if we want to limit the selection set to models with 2D rendering or not
        /// </summary>
        public bool LimitToModelsWith2DRendering { get; set; }

        /// <summary>
        /// The parent tax id of any sequences with secondary structure models included in the set
        /// </summary>
        public int ParentTaxId { get; set; }

        /// <summary>
        /// RNA location identifier
        /// </summary>
        public int LocationId { get; set; }

        /// <summary>
        /// Sequence type (5s, 16s, 23s)
        /// </summary>
        public int SequenceTypeId { get; set; }

        /// <summary>
        /// Returns whether this filter + DB connection is valid.
        /// </summary>
        public bool IsValid
        {
            get
            {
                return Connection != null && Connection.IsValid && SequenceTypeId > 0 && AlignmentId > 0 &&
                       !string.IsNullOrEmpty(Name) && (ParentTaxId > 0 || SelectedSet.Count > 0);
            }
        }

        /// <summary>
        /// The rCAD database connection parameters
        /// </summary>
        public RcadConnection Connection { get; set; }

        public StructureModelSet()
        {
            Connection = new RcadConnection();
            SelectedSet = new List<int>();
        }
    }
}
