﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using Bio.Data.Interfaces;

namespace Bio.Data.Providers.Structure
{
    public class RNASecondaryStructureBasePair : BioExtensiblePropertyBase, IBasePairBioEntity
    {
        #region IBasePairBioEntity Members

        public int FivePrimeIndex
        {
            get { return _fivePrimeIndex; }
        }

        public IBioSymbol FivePrimeNucleotide
        {
            get { return _sequence.SequenceData[FivePrimeIndex]; }
        }

        public int ThreePrimeIndex
        {
            get { return _threePrimeIndex; }
        }

        public IBioSymbol ThreePrimeNucleotide
        {
            get { return _sequence.SequenceData[ThreePrimeIndex]; }
        }

        public string Text
        {
            get { return string.Format("{0}-{1}", FivePrimeNucleotide.Value, ThreePrimeNucleotide.Value); }
        }

        #endregion

        private IBioSequence _sequence;
        private int _fivePrimeIndex;
        private int _threePrimeIndex;

        public RNASecondaryStructureBasePair(IBioSequence sequence, int fivePrimeIndex, int threePrimeIndex)
        {
            _fivePrimeIndex = fivePrimeIndex;
            _threePrimeIndex = threePrimeIndex;
            _sequence = sequence;
        }
    }
}
