﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Bio.Data.Interfaces.Drawing;

namespace Bio.Data.Providers.Structure.XRNA.Internal
{
    internal class XRNALineShape : ILineShape
    {
        public static XRNALineShape Create(string xrnaLine)
        {
            if (xrnaLine != null)
            {
                if (Regex.IsMatch(xrnaLine, BPLineConnectorDefinition))
                {
                    string[] tokens = Regex.Split(xrnaLine, " ");
                    Debug.Assert(tokens.Length == 8);
                    double thickness;
                    if (Double.TryParse(tokens[1], out thickness))
                    {
                        /*XRNALineShape shape = new XRNALineShape()
                        {
                            Thickness = thickness,
                            Color = ("#" + tokens[2]).PadLeft(7, '0')
                        };*/
                        XRNALineShape shape = new XRNALineShape()
                        {
                            Thickness = thickness,
                            Color = "#" + tokens[2].PadLeft(6, '0')
                        };
                        return shape;
                    }
                }
            }
            return null; //Some error was encountered or this was not a valid line. In the future, we would probably throw an error to distinguish
        }

        #region ILineShape Members

        public double Thickness
        {
            get;
            set;
        }

        public string Color
        {
            get;
            set;
        }

        #endregion

        #region Private Methods and Members

        private XRNALineShape()
        {
            Thickness = 1.0;
            Color = "#000000";
        }

        private static string BPLineConnectorDefinition = @"!Bl\s\d+.\d{2}\s[0-9a-fA-F]+(\s\d){5}";

        #endregion
    }
}
