﻿/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;

namespace Bio.Data.Interfaces
{
    /// <summary>
    /// An RNA secondary structure base pair
    /// </summary>
    public interface IBasePairBioEntity
    {
        /// <summary>
        /// The index (within the sequence) of the five prime nucleotide in the base pair
        /// </summary>
        int FivePrimeIndex { get; }

        /// <summary>
        /// The five prime nucleotide in the base pair
        /// </summary>
        IBioSymbol FivePrimeNucleotide { get; }

        /// <summary>
        /// The index (within the sequence) of the three prime nucleotide in the base pair
        /// </summary>
        int ThreePrimeIndex { get; }

        /// <summary>
        /// The three prime nucleotide in the base pair
        /// </summary>
        IBioSymbol ThreePrimeNucleotide { get; }

        /// <summary>
        /// Textual representation of the base pair
        /// </summary>
        string Text { get; }
    }

    public class BasePairComparer : IEqualityComparer<IBasePairBioEntity>
    {
        #region IEqualityComparer<IBasePairBioEntity> Members

        public bool Equals(IBasePairBioEntity x, IBasePairBioEntity y)
        {
            if (x == null || y == null) return false;
            if (x.FivePrimeNucleotide == null || x.ThreePrimeNucleotide==null) return false;
            return ((x.FivePrimeIndex == y.FivePrimeIndex) && 
                    (x.ThreePrimeIndex == y.ThreePrimeIndex) &&
                    (x.FivePrimeNucleotide == y.FivePrimeNucleotide) &&
                    (x.ThreePrimeNucleotide == y.FivePrimeNucleotide)) ? true : false;
        }

        public int GetHashCode(IBasePairBioEntity obj)
        {
            return obj.GetHashCode();
        }

        #endregion
    }
}
