﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Windows;
using Bio.Data.Interfaces;

namespace Bio.Views.Structure.Editor.Internal.Data
{
    internal class LocalNucleotideSequenceVector : IVector
    {
        #region IVector

        public double RayAngle { get; set; }
        public double Magnitude { get; set; }
        public double StartX { get; set; }
        public double StartY { get; set; }

        public IVector Clone()
        {
            return new LocalNucleotideSequenceVector(this);
        }

        #endregion

        public LocalNucleotideSequenceVector()
        {
        }

        public LocalNucleotideSequenceVector(Point startPoint, Point endPoint)
        {
            CalculateVectorFromPoints(startPoint, endPoint);
        }

        public LocalNucleotideSequenceVector(Point location)
        {
            this.StartX = location.X;
            this.StartY = location.Y;
        }

        protected LocalNucleotideSequenceVector(LocalNucleotideSequenceVector toCopy)
        {
            this.RayAngle = toCopy.RayAngle;
            this.Magnitude = toCopy.Magnitude;
            this.StartX = toCopy.StartX;
            this.StartY = toCopy.StartY;
        }

        private void CalculateVectorFromPoints(Point startPoint, Point endPoint)
        {
            StartX = startPoint.X;
            StartY = startPoint.Y;
            RayAngle = Math.PI + Math.Atan2((StartY - endPoint.Y), (StartX - endPoint.X));
            Magnitude = Math.Sqrt(Math.Pow((StartY - endPoint.Y), 2) + Math.Pow((StartX - endPoint.X), 2));
        }
    }
}
