﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

namespace Bio.Views.Structure.Editor.Internal.Data
{
    public class StructureModelData
    {
        public EditableStructureModel Entity { get; set; }

        public bool IsLinkable
        {
            get
            {
                return (Entity != null &&
                        Entity.Sequence != null &&
                        Entity.Sequence.ScientificName != null &&
                        !Entity.Sequence.ScientificName.Equals(string.Empty) &&
                        Entity.Sequence.Id != null &&
                        !Entity.Sequence.Id.Equals(string.Empty) &&
                        Entity.Sequence.SourceId != null &&
                        !Entity.Sequence.SourceId.Equals(string.Empty)) ? true : false;
            }
        }
        
        public string Title { get; set; }
        public string Key { get; set; }
        public string Label { get; set; }
        public bool InMemoryDataOnly { get; set; }

        public string SequenceId
        {
            get 
            {
                if (IsLinkable)
                    return Entity.Sequence.Id;
                else
                    return string.Empty;
            }
        }

        public string AlignmentId
        {
            get
            {
                if (IsLinkable)
                    return Entity.Sequence.SourceId;
                else
                    return string.Empty;
            }
        }

        public string ScientificName
        {
            get
            {
                if (IsLinkable)
                    return Entity.Sequence.ScientificName;
                else
                    return string.Empty;
            }
        }

        public override bool Equals(object obj)
        {
            if (obj == null) return false;
            StructureModelData toTest = obj as StructureModelData;
            if (toTest != null)
            {
                return (this.Entity == toTest.Entity) ? true : false;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return Entity.GetHashCode();
        }
    }
}
