﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Diagnostics;
using System.Windows;
using System.Windows.Media;
using Bio.Data;
using Bio.Data.Providers.Structure;
using Bio.Views.Structure.Editor.Internal.Rendering;
using Bio.Views.Structure.Editor.Internal.Rendering.Visuals;

namespace Bio.Views.Structure.Editor.Internal.Editing
{
    public class NucleotideVisualComponentPropertyContext : StructureElementVisualComponentPropertyContext
    {
        //Directly apply the latest property context state to the visual (this is basically how we support undo, assuming the "latest" property context was rolled back to reflect the undo).
        public override void Apply()
        {
            NtVisual.NucleotideFontFamily = NucleotideFontFamily;
            NtVisual.NucleotideFontStyle = NucleotideFontStyle;
            NtVisual.NucleotideFontWeight = NucleotideFontWeight;
            NtVisual.NucleotideFontSize = NucleotideFontSize;
            NtVisual.NucleotideFill = NucleotideFill;
            NtVisual.NucleotideRenderingStyle = NucleotideRenderingStyle;
            NtVisual.NucleotideSubstitution = NucleotideSubstitution;
            NtVisual.IsNucleotideSubstituted = IsNucleotideSubstituted;
            NtVisual.NucleotideCircleDiameter = NucleotideCircleDiameter;
            NtVisual.NucleotideCircleEdgeThickness = NucleotideCircleEdgeThickness;
            NtVisual.NucleotideCircleFill = NucleotideCircleFill;
            NtVisual.NucleotideCircleIsFilled = NucleotideCircleIsFilled;
            NtVisual.NucleotideCircleStroke = NucleotideCircleStroke;
        }

        //Save the latest property context state to a new metadata object.
        public override object Save()
        {
            Debug.Assert(NtVisual != null);
            Debug.Assert(NtVisual.Data != null);
            if (NtVisual.IsDirty())
            {
                NucleotideMetadata ntDataObjMetadata = new NucleotideMetadata();
                SaveNucleotideRendering(ntDataObjMetadata);
                return ntDataObjMetadata;
            }
            return null;
        }

        //Save the latest property context state to the data object.
        /*public override void Save()
        {
            Debug.Assert(NtVisual != null);
            Debug.Assert(NtVisual.Data != null);
            if (NtVisual.IsDirty())
            {
                IExtensibleProperties ntDataObjExt = NtVisual.Data as IExtensibleProperties;
                if (ntDataObjExt != null)
                {
                    NucleotideMetadata ntDataObjMetadata = (ntDataObjExt.DoesExtendedPropertyExist(NucleotideMetadata.Key)) ? ntDataObjExt.GetExtendedProperty<NucleotideMetadata>(NucleotideMetadata.Key) :
                        new NucleotideMetadata();

                    SaveNucleotideRendering(ntDataObjMetadata);

                    if (!ntDataObjExt.DoesExtendedPropertyExist(NucleotideMetadata.Key))
                        ntDataObjExt.SetExtendedProperty(NucleotideMetadata.Key, ntDataObjMetadata);
                }
                else
                {
                    throw new NotSupportedException("Error: Saving Nucleotide Visual Context is not possible as the Data Object does not support Metadata");
                }
            }
        }*/

        public NucleotideVisualComponentPropertyContext(NucleotideStructureElementVisualComponent ntVisual)
            : base(ntVisual.IsComponentVisible)
        {
            Debug.Assert(ntVisual != null);
            Debug.Assert(ntVisual.Data != null);
            NtVisual = ntVisual;
            //We set the initial context directly to avoid tracking the initial state as edits.
            CurrentContext.NucleotideFontFamily = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideFontFamilyProperty) as FontFamily;
            CurrentContext.NucleotideFontStyle = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideFontStyleProperty) as FontStyle?;
            CurrentContext.NucleotideFontWeight = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideFontWeightProperty) as FontWeight?;
            CurrentContext.NucleotideFontSize = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideFontSizeProperty) as double?;
            CurrentContext.NucleotideFill = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideFillProperty) as Brush;
            CurrentContext.NucleotideRenderingStyle = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideRenderingStyleProperty) as NucleotideRenderingMode?;
            CurrentContext.NucleotideSubstitution = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideSubstitutionProperty) as char?;
            CurrentContext.IsNucleotideSubstituted = NtVisual.GetValue(NucleotideStructureElementVisualComponent.IsNucleotideSubstitutedProperty) as bool?;
            CurrentContext.NucleotideCircleDiameter = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideCircleDiameterProperty) as double?;
            CurrentContext.NucleotideCircleEdgeThickness = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideCircleEdgeThicknessProperty) as double?;
            CurrentContext.NucleotideCircleFill = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideCircleFillProperty) as Brush;
            CurrentContext.NucleotideCircleIsFilled = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideCircleIsFilledProperty) as bool?;
            CurrentContext.NucleotideCircleStroke = NtVisual.GetValue(NucleotideStructureElementVisualComponent.NucleotideCircleStrokeProperty) as Brush;
        }

        private NucleotideStructureElementVisualComponent NtVisual { get; set; }

        private void SaveRenderingStyle(NucleotideMetadata metadata)
        {
            if (NucleotideRenderingStyle.HasValue)
            {
                switch (NucleotideRenderingStyle.Value)
                {
                    case NucleotideRenderingMode.Text:
                        metadata.Style = NucleotideMetadata.RenderingMode.Text;
                        break;
                    case NucleotideRenderingMode.Circle:
                        metadata.Style = NucleotideMetadata.RenderingMode.Circle;
                        break;
                    default:
                        metadata.Style = null;
                        break;
                };
            }
            else { metadata.Style = null; }
        }

        private void SaveNucleotideRendering(NucleotideMetadata ntDataObjMetadata)
        {
            SaveRenderingStyle(ntDataObjMetadata);

            if (ContextHasTextRenderingPropertiesSet())
            {
                if (ntDataObjMetadata.TextRendering == null)
                    ntDataObjMetadata.TextRendering = new RNASecondaryStructureNucleotideTextShape();

                ntDataObjMetadata.TextRendering.Font = (NucleotideFontFamily != null) ? NucleotideFontFamily.Source : null;
                ntDataObjMetadata.TextRendering.FontStyle = (NucleotideFontStyle.HasValue) ? NucleotideFontStyle.Value.ToString() : null;
                ntDataObjMetadata.TextRendering.FontWeight = (NucleotideFontWeight.HasValue) ? NucleotideFontWeight.Value.ToString() : null;
                ntDataObjMetadata.TextRendering.FontSize = (NucleotideFontSize.HasValue) ? NucleotideFontSize.Value : 0.0;
                ntDataObjMetadata.TextRendering.Color = (NucleotideFill != null) ? NucleotideFill.ToString() : null;
            }
            else
            {
                ntDataObjMetadata.TextRendering = null;
            }

            if (ContextHasCircleRenderingPropertiesSet())
            {
                if (ntDataObjMetadata.CircleRendering == null)
                    ntDataObjMetadata.CircleRendering = new RNASecondaryStructureNucleotideCircleShape();

                ntDataObjMetadata.CircleRendering.EdgeThickness = (NucleotideCircleEdgeThickness.HasValue && NucleotideCircleEdgeThickness.Value > 0.0) ? NucleotideCircleEdgeThickness.Value : 0.0;
                ntDataObjMetadata.CircleRendering.Diameter = (NucleotideCircleDiameter.HasValue && NucleotideCircleDiameter.Value > 0.0) ? NucleotideCircleDiameter.Value : 0.0;
                ntDataObjMetadata.CircleRendering.Filled = NucleotideCircleIsFilled;
                ntDataObjMetadata.CircleRendering.FillColor = (NucleotideCircleFill != null) ? NucleotideCircleFill.ToString() : null;
                ntDataObjMetadata.CircleRendering.EdgeColor = (NucleotideCircleStroke != null) ? NucleotideCircleStroke.ToString() : null;
            }
            else
            {
                ntDataObjMetadata.CircleRendering = null;
            }
        }
    }
}
