﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Diagnostics;
using System.Windows;
using System.Windows.Media;

namespace Bio.Views.Structure.Editor.Internal.Rendering.Visuals
{
    public class HelixDrawingVisual : DrawingVisual
    {
        public Rect Measure()
        {
            if (Container.IsPseudoknotted)
                return Rect.Empty;
            Debug.Assert(Container.Edges.Count == 4);
            PathGeometry helixPath = new PathGeometry();
            PathFigure figure = new PathFigure();
            figure.StartPoint = Container.Edges[0];
            figure.IsClosed = true;
            figure.Segments.Add(new LineSegment(Container.Edges[1], true));
            figure.Segments.Add(new LineSegment(Container.Edges[2], true));
            figure.Segments.Add(new LineSegment(Container.Edges[3], true));
            helixPath.Figures.Add(figure);
            RenderingGeometry = helixPath;
            return RenderingGeometry.Bounds;
        }

        public void Render()
        {
            //Initially, we are only going to "render" anything when we are selected.
            //A bounding box though the points that can be used to show a rotation angle.
            using (var drawingContext = RenderOpen())
            {
                if (Container.IsSelected && !Container.IsPseudoknotted)
                {
                    if(!Container.HideBoundingBox)
                        drawingContext.DrawGeometry(null, new Pen(Container.BoundingBoxStroke, Container.BoundingBoxStrokeThickness), RenderingGeometry);
                }
            }
        }

        public HelixDrawingVisual(HelixStructureElementVisualComponent container)
        {
            Debug.Assert(container != null);
            Container = container;
        }

        private HelixStructureElementVisualComponent Container { get; set; }
        private Geometry RenderingGeometry { get; set; }
    }
}
