﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Bio.Data;
using Bio.Data.Interfaces;
using JulMar.Windows.Mvvm;

namespace Bio.Views.Structure.Editor.ViewModels
{
    public class BuildBioEntityTypeFilterViewModel : SimpleViewModel
    {
        public ObservableCollection<FilterDataItem> Nucleotides { get; private set; }
        public ObservableCollection<FilterDataItem> BasePairs { get; private set; }

        public IEnumerable<IBioSymbol> NucleotidesInFilter
        {
            get { return Nucleotides.Where(fdi => fdi.Include).Select(fdi => fdi.Nucleotide); }
        }

        public IEnumerable<string> BasePairsInFilter
        {
            get { return BasePairs.Where(fdi => fdi.Include).Select(fdi => fdi.BasePair); }
        }

        public BuildBioEntityTypeFilterViewModel()
        {
            Nucleotides = new ObservableCollection<FilterDataItem>()
            {
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'A'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'G'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'C'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'U'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'T'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'R'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'Y'), Include = false },
                new FilterDataItem() { Nucleotide = new BioSymbol(BioSymbolType.Nucleotide, 'N'), Include = false }
            };

            BasePairs = new ObservableCollection<FilterDataItem>();
            foreach (var nt in Nucleotides)
            {
                foreach (var pairedNt in Nucleotides)
                {
                    BasePairs.Add(new FilterDataItem() { BasePair = string.Format("{0}-{1}", nt.Nucleotide.Value, pairedNt.Nucleotide.Value), Include = false });
                }
            }
        }

        public class FilterDataItem
        {
            public IBioSymbol Nucleotide { get; set; }
            public string BasePair { get; set; }
            public bool Include { get; set; }
        }
    }
}
