﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using Bio.Views.Structure.Editor.Internal.Validations;
using JulMar.Windows.Validations;

namespace Bio.Views.Structure.Editor.ViewModels
{
    public class PromptForStructureElementTranslationViewModel : ValidatingViewModel
    {
        [PositiveOrZeroValue]
        public double LocationX
        {
            get { return _locationX; }
            set
            {
                _locationX = value;
                OnPropertyChanged("LocationX", "IsValid");
            }
        }

        [PositiveOrZeroValue]
        public double LocationY
        {
            get { return _locationY; }
            set
            {
                _locationY = value;
                OnPropertyChanged("LocationY", "IsValid");
            }
        }

        public double DeltaX
        {
            get { return _deltaX; }
            set
            {
                _deltaX = value;
                OnPropertyChanged("DeltaX", "IsValid");
            }
        }

        public double DeltaY
        {
            get { return _deltaY; }
            set
            {
                _deltaY = value;
                OnPropertyChanged("DeltaY", "IsValid");
            }
        }

        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                OnPropertyChanged("Title");
            }
        }

        public string Instructions
        {
            get { return _instructions; }
            set
            {
                _instructions = value;
                OnPropertyChanged("Instructions");
            }
        }

        public bool WantDeltas
        {
            get { return _wantDeltas; }
            set
            {
                _wantDeltas = value;
                OnPropertyChanged("WantDeltas", "IsValid");
            }
        }

        public bool IsValid
        {
            get
            {
                if (WantDeltas) { return true; }
                else { return (LocationX >= 0 && LocationY >= 0) ? true : false; }
            }
        }

        private bool _wantDeltas;
        private string _instructions;
        private string _title;
        private double _locationX;
        private double _locationY;
        private double _deltaX;
        private double _deltaY;
    }
}
