﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Bio.Views.Structure.Editor.ViewModels;
using JulMar.Windows.Interfaces;
using JulMar.Windows.Mvvm;

namespace Bio.Views.Structure.Editor.Views
{
    /// <summary>
    /// Interaction logic for SecondaryStructureBirdsEyeViewer.xaml
    /// </summary>
    public partial class SecondaryStructureBirdsEyeViewer : UserControl
    {
        private bool _inPanMode;
        private Point _startPanPosition;

        public SecondaryStructureBirdsEyeViewer()
        {
            Loaded += new RoutedEventHandler(SecondaryStructureBirdsEyeViewer_Loaded);
            InitializeComponent();
        }

        void SecondaryStructureBirdsEyeViewer_Loaded(object sender, RoutedEventArgs e)
        {
            var mediator = ViewModel.ServiceProvider.Resolve<IMessageMediator>();
            if (mediator != null)
                mediator.RegisterHandler<SecondaryStructureEditorViewModel>(SecondaryStructureEditorViewMessages.SecondaryStructureEditorViewChangedMessage, OnSecondaryStructureEditorViewChanged);
        }

        private void OnSecondaryStructureEditorViewChanged(SecondaryStructureEditorViewModel vm)
        {
            DataContext = vm.BirdsEyeViewModel;
        }

        protected override void OnMouseLeave(MouseEventArgs e)
        {
            if (_inPanMode)
            {
                _inPanMode = false;
                SecondaryStructureEditorBirdsEyeViewModel vm = DataContext as SecondaryStructureEditorBirdsEyeViewModel;
                vm.UpdateEditorPosition();
            }
            base.OnMouseLeave(e);
        }

        protected override void OnPreviewMouseLeftButtonUp(MouseButtonEventArgs e)
        {
            if (_inPanMode)
            {
                SelectorMovedTo(e.GetPosition(bevCanvas));
                _inPanMode = false;
                SecondaryStructureEditorBirdsEyeViewModel vm = DataContext as SecondaryStructureEditorBirdsEyeViewModel;
                vm.UpdateEditorPosition();
            }
            base.OnPreviewMouseLeftButtonUp(e);
        }

        protected override void OnPreviewMouseMove(MouseEventArgs e)
        {
            if (!bevSelector.IsMouseOver)
            {
                _inPanMode = false;
                return;
            }

            if (_inPanMode)
            {
                SelectorMovedTo(e.GetPosition(bevCanvas));
                e.Handled = true;
            }
            base.OnPreviewMouseMove(e);
        }

        protected override void OnPreviewMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            if (bevSelector.IsMouseOver)
            {
                _inPanMode = true;
                _startPanPosition = e.GetPosition(bevCanvas);
                e.Handled = true;
            }
            base.OnPreviewMouseLeftButtonDown(e);
        }

        private void SelectorMovedTo(Point end)
        {
            Point currentPosition = end;
            Point delta = new Point(currentPosition.X - _startPanPosition.X,
                currentPosition.Y - _startPanPosition.Y);
            Point selectorTopLeft = new Point(Canvas.GetLeft(bevSelector), Canvas.GetTop(bevSelector));
            Point maxSelectorTopLeft = new Point(bevCanvas.Width - bevSelector.Width, bevCanvas.Height - bevSelector.Height);
            Point newSelectorTopLeft = new Point();

            if (selectorTopLeft.X + delta.X > maxSelectorTopLeft.X)
            {
                newSelectorTopLeft.X = maxSelectorTopLeft.X;
            }
            else if (selectorTopLeft.X + delta.X < 0)
            {
                newSelectorTopLeft.X = 0;
            }
            else
            {
                newSelectorTopLeft.X = selectorTopLeft.X + delta.X;
            }

            if (selectorTopLeft.Y + delta.Y > maxSelectorTopLeft.Y)
            {
                newSelectorTopLeft.Y = maxSelectorTopLeft.Y;
            }
            else if (selectorTopLeft.Y + delta.Y < 0)
            {
                newSelectorTopLeft.Y = 0;
            }
            else
            {
                newSelectorTopLeft.Y = selectorTopLeft.Y + delta.Y;
            }

            _startPanPosition = currentPosition;
            Canvas.SetLeft(bevSelector, newSelectorTopLeft.X);
            Canvas.SetTop(bevSelector, newSelectorTopLeft.Y);
        }
    }
}
