﻿/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bio.Data.Interfaces;

namespace Bio.Data.Providers.Structure
{
    public class RNASecondaryStructureModel : BioExtensiblePropertyBase, IStructureModelBioEntity
    {
        #region IStructureModelBioEntity Members

        public IBioSequence Sequence
        {
            get { return _sequence; }
        }

        public IList<IBasePairBioEntity> BasePairs
        {
            get { return _basePairs; }
            set { _basePairs = value; }
        }

        public IList<IStructureExtentBioEntity> Extents
        {
            get { return _structureExtents; }
            set { _structureExtents = value; }
        }

        #endregion

        private IList<IBasePairBioEntity> _basePairs;
        private IList<IStructureExtentBioEntity> _structureExtents;
        private IBioSequence _sequence;

        public RNASecondaryStructureModel(string name, IBioSequence sequence)
        {
            _sequence = sequence;
            _structureExtents = new List<IStructureExtentBioEntity>();
            _basePairs = new List<IBasePairBioEntity>();
        }
    }
}
