﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using Bio.Data.Interfaces.Drawing;

namespace Bio.Data.Providers.Structure.XRNA.Internal
{
    internal class XRNATextShape : ITextShape
    {
        /// <summary>
        /// TextRendering with specific properties
        /// </summary>
        /// <param name="typefaceID"></param>
        /// <param name="fontSize"></param>
        /// <param name="color"></param>
        /// <returns></returns>
        public static XRNATextShape Create(int typefaceID, double fontSize, string color)
        {
            return new XRNATextShape(typefaceID, fontSize, color);
        }

        /*public static XRNATextShape Create(string xrnaLine)
        {
            return null; //Some error was encountered or this was not a valid line. In the future, we would probably throw an error to distinguish
        }*/

        /// <summary>
        /// Default TextRendering:
        /// Font = "Helvetical, Arial"
        /// FontStyle = "Normal"
        /// FontWeight = "Regular"
        /// Color = Black
        /// Size = 8.0pt
        /// </summary>
        /// <returns></returns>
        public static XRNATextShape CreateDefault()
        {
            return new XRNATextShape(0, 8.0, "#000000");
        }

        #region ITextShape Members

        public string Font
        {
            get;
            set;
        }

        public double FontSize
        {
            get;
            set;
        }

        public string FontStyle
        {
            get;
            set;
        }

        public string FontWeight
        {
            get;
            set;
        }

        public string Color
        {
            get;
            set;
        }

        #endregion

        #region IShape Members

        #endregion

        #region Private Members and Methods

        private XRNATextShape(int typefaceID, double fontSize, string color)
        {
            XRNAFile.XRNATypeface typeface = XRNAFile.ParseXRNACode(typefaceID);
            Font = typeface.Font;
            FontStyle = typeface.FontStyle;
            FontWeight = typeface.FontWeight;
            FontSize = fontSize;
            Color = color;
        }

        #endregion
    }
}
