﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Collections.Generic;
using Bio.Data.Interfaces.Drawing;

namespace Bio.Data.Interfaces
{
    /// <summary>
    /// This represents an RNA secondary structure model that has base pairs, helices
    /// and loops. It can also have 2-D drawing information, labels, shapes, etc...
    /// 
    /// A secondary structure model may have an absolute rendering provided as an
    /// extensible property. If it is available, it will be used for rendering.
    /// 
    /// If a rendering is provided, the cooridinates are expected to be associated
    /// with the nucleotides
    /// </summary>
    public interface IStructureModelBioEntity
    {
        /// <summary>
        /// The underlying sequence
        /// </summary>
        IBioSequence Sequence { get; }

        /// <summary>
        /// The base pairs in the RNA secondary structure model.
        /// </summary>
        IList<IBasePairBioEntity> BasePairs { get; }

        /// <summary>
        /// The extents in the RNA secondary structure model. Currently this is loops, helices, pseudoknots.
        /// In the future, it will be extended to motifs.
        /// </summary>
        IList<IStructureExtentBioEntity> Extents { get; }
    }
}
