﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using Bio.Views.Structure.Editor.Internal.Validations;
using JulMar.Windows.Validations;

namespace Bio.Views.Structure.Editor.ViewModels
{
    public class PromptForSizingViewModel : ValidatingViewModel
    {
        public bool HaveWidth
        {
            get { return _haveWidth; }
            set
            {
                _haveWidth = value;
                OnPropertyChanged("HaveWidth", "IsValid");
            }
        }

        public bool HavePadding
        {
            get { return _havePadding; }
            set
            {
                _havePadding = value;
                OnPropertyChanged("HavePadding", "IsValid");
            }
        }

        public string WidthLabel
        {
            get { return _widthLabel; }
            set
            {
                _widthLabel = value;
                OnPropertyChanged("WidthLabel");
            }
        }

        public string PaddingLabel
        {
            get { return _paddingLabel; }
            set
            {
                _paddingLabel = value;
                OnPropertyChanged("PaddingLabel");
            }
        }

        [PositiveValue]
        public double Width
        {
            get { return _width; }
            set
            {
                _width = value;
                OnPropertyChanged("Width", "IsValid");
            }
        }

        [PositiveValue]
        public double Padding
        {
            get { return _padding; }
            set
            {
                _padding = value;
                OnPropertyChanged("Padding", "IsValid");
            }
        }

        public string Instructions
        {
            get { return _instructions; }
            set
            {
                _instructions = value;
                OnPropertyChanged("Instructions");
            }
        }

        public string Title
        {
            get { return _title; }
            set
            {
                _title = value;
                OnPropertyChanged("Title");
            }
        }

        public bool IsValid
        {
            get { return ((HavePadding && Padding > 0.0) || (HaveWidth && Width > 0.0)) ? true : false; }
        }

        public PromptForSizingViewModel()
        {
            Padding = 8.0;
            Width = 20.0;
            HavePadding = false;
            HaveWidth = false;
            WidthLabel = "Width";
            PaddingLabel = "Padding";
        }

        private bool _haveWidth;
        private bool _havePadding;
        private double _padding;
        private double _width;
        private string _title;
        private string _instructions;
        private string _widthLabel;
        private string _paddingLabel;
    }
}
