﻿/* 
* Copyright (c) 2010, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Bio.Views.Structure.Editor.Internal.Commands;
using Bio.Views.Structure.Editor.ViewModels;
using JulMar.Windows.Interfaces;
using JulMar.Windows.Mvvm;

namespace Bio.Views.Structure.Editor.Views
{
    /// <summary>
    /// Interaction logic for SecondaryStructureEditorVisualComponentInspector.xaml
    /// </summary>
    public partial class SecondaryStructureEditorVisualComponentInspector : UserControl
    {
        public SecondaryStructureEditorVisualComponentInspector()
        {
            _registeredEditorViews = new List<SecondaryStructureEditorViewModel>();
            Loaded += new RoutedEventHandler(SecondaryStructureEditorVisualComponentInspector_Loaded);
            InitializeComponent();

            var mediator = ViewModel.ServiceProvider.Resolve<IMessageMediator>();
            if(mediator != null)
                mediator.RegisterHandler<SecondaryStructureEditorViewModel>(SecondaryStructureEditorViewMessages.SecondaryStructureEditorViewClosingMessage, OnSecondaryStructureEditorViewClosing);

            CommandBindings.Add(new CommandBinding(StructureEditorCommands.ChangeNucleotideFill, StructureEditorCommands.OnChangeNucleotideFill, StructureEditorCommands.OnCanEditVisualComponentBrush));
            CommandBindings.Add(new CommandBinding(StructureEditorCommands.ChangeNucleotideCircleFill, StructureEditorCommands.OnChangeNucleotideCircleFill, StructureEditorCommands.OnCanEditVisualComponentBrush));
            CommandBindings.Add(new CommandBinding(StructureEditorCommands.ChangeNucleotideCircleStroke, StructureEditorCommands.OnChangeNucleotideCircleStroke, StructureEditorCommands.OnCanEditVisualComponentBrush));
            CommandBindings.Add(new CommandBinding(StructureEditorCommands.ChangeBasePairConnectorFill, StructureEditorCommands.OnChangeBasePairConnectorFill, StructureEditorCommands.OnCanEditVisualComponentBrush));
            CommandBindings.Add(new CommandBinding(StructureEditorCommands.ChangeBasePairConnectorStroke, StructureEditorCommands.OnChangeBasePairConnectorStroke, StructureEditorCommands.OnCanEditVisualComponentBrush));
        }

        private void SecondaryStructureEditorVisualComponentInspector_Loaded(object sender, RoutedEventArgs e)
        {
            var mediator = ViewModel.ServiceProvider.Resolve<IMessageMediator>();
            if (mediator != null)
                mediator.RegisterHandler<SecondaryStructureEditorViewModel>(SecondaryStructureEditorViewMessages.SecondaryStructureEditorViewChangedMessage, OnSecondaryStructureEditorViewChanged);
        }

        private void OnSecondaryStructureEditorViewChanged(SecondaryStructureEditorViewModel vm)
        {
            if (vm != null)
            {
                if (!_registeredEditorViews.Contains(vm))
                    _registeredEditorViews.Add(vm);

                DataContext = vm.PropertyInspectorViewModel;
                vm.PropertyInspectorViewModel.TriggerRefresh();
            }
        }

        private void OnSecondaryStructureEditorViewClosing(SecondaryStructureEditorViewModel vm)
        {
            if (_registeredEditorViews.Contains(vm))
            {
                _registeredEditorViews.Remove(vm);
            }

            if (_registeredEditorViews.Count == 0)
                DataContext = new SecondaryStructureEditorVisualComponentInspectorViewModel();
        }

        private List<SecondaryStructureEditorViewModel> _registeredEditorViews;
    }
}
