﻿/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace TaxonomyLoader
{
    public class Name
    {
        public static IEnumerable<Name> Load(string namesFileName)
        {
            try
            {
                string allnames = File.ReadAllText(namesFileName);
                IEnumerable<Name> retValue = from Match name in Regex.Matches(allnames, NameFormat)
                                             select new Name()
                                             {
                                                 TaxID = Int32.Parse(name.Groups["TaxID"].Value),
                                                 NameText = name.Groups["NameTxt"].Value.Trim(),
                                                 NameClass = name.Groups["NameClass"].Value.Trim()
                                             };
                return retValue;
            }
            catch (Exception)
            {
                return new List<Name>();
            }
        }

        public int TaxID { get; set; }
        public string NameText { get; set; }
        public string NameClass { get; set; }

        private static string NameFormat = @"(?<TaxID>[0-9]+)\t\|\t(?<NameTxt>[^\t\|]*)\t\|\t(?<UniqueName>[^\t\|]*)\t\|\t(?<NameClass>[^\t\|]*)\t\|\n";
    }
}
