﻿/* 
* Copyright (c) 2009, The University of Texas at Austin
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification, 
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, 
* this list of conditions and the following disclaimer in the documentation and/or other materials 
* provided with the distribution.
*
* Neither the name of The University of Texas at Austin nor the names of its contributors may be 
* used to endorse or promote products derived from this software without specific prior written 
* permission.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
* FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
* THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;

namespace TaxonomyLoader
{
    public class Node
    {
        public static IEnumerable<Node> Load(string nodesFileName)
        {
            try
            {
                //Just load the whole thing into memory...shouldn't be an issue
                string allnodes = File.ReadAllText(nodesFileName);
                IEnumerable<Node> retValue = from Match node in Regex.Matches(allnodes, NodeFormat)
                                             select new Node()
                                             {
                                                 TaxID = Int32.Parse(node.Groups["TaxID"].Value),
                                                 ParentTaxID = Int32.Parse(node.Groups["ParentTaxID"].Value)
                                             };
                return retValue;
            }
            catch (Exception)
            {
                return new List<Node>();
            }
        }

        public int TaxID { get; set; }
        public int ParentTaxID { get; set; }

        private static string NodeFormat = @"(?<TaxID>[0-9]+)\t\|\t(?<ParentTaxID>[0-9]+)\t\|\t(?<Rank>[^\t\|]+)\t\|\t(?<EMBLCode>[^\t\|]*)\t\|\t(?<DivID>[0-9]+)\t\|\t(?<InheritedDivFlag>[0|1])\t\|\t(?<GenCodeID>[0-9]+)\t\|\t(?<InheritedGCFlag>[0|1])\t\|\t(?<MitoGenCodeID>[0-9]+)\t\|\t(?<InheritedMCGCFlag>[0|1])\t\|\t(?<GBHiddenFlag>[0|1])\t\|\t(?<HiddenRootSubtreeFlag>[0|1])\t\|\t(?<Comments>[^\t\|]*)\t\|\n";
    }
}
